%% BasicSpectrometer

% Copyright 2025 by Avenir Photonics GmbH & Co. KG

% This is a simple demo to show how to use the spectrometer from Matlab.
% The spectrometer is controlled via USB using the ZioLinkLibrary.dll.
% If you are using an earlier version of Matlab, ...

%% IMPORTANT NOTE

% The function NET.addAssembly() needs the full path of the library file
% 'ZioLinkLibrary.dll'. Before running this demo code, set the absolute
% path to the DLL file below.

%% Load the .NET library

disp("BasicSpectrometer")
disp("-----------------")

try
    a = NET.addAssembly('c:\....INSERT CORRECT FILE PATH HERE ....\BasicSpectrometerDLL Matlab\ZioLinkLibrary.dll');
catch e
    e.message
    if(isa(e,'NET.NetException'))
        e.ExceptionObject
    end
end

%% Open USB connection to spectrometer

% Close any previously opened device
if exist('spectr', 'var')
    if ~isempty(spectr)
        spectr.Close()
    end
end

disp('Searching for spectrometer:')
devices = DeviceClasses.ZioLinkSpectrometer.FindDevices();

if devices.Length == 0
    error 'No device found.';
end

spectr = devices(1);

try
    spectr.Open(); % open connection to the device

    %% Read device information

    fprintf('Model name: %s\n', char(spectr.DeviceName)');
    fprintf('Serial number: %s\n', char(spectr.SerialNumber)');

    pixelCount = spectr.PixelCount;
    fprintf('Number of pixels: %d\n', pixelCount');

    %% Read wavelengths
    
    wavelengths = spectr.Wavelengths();
    fprintf('Wavelength range: %.2f to %.2f nm\n', wavelengths(1), wavelengths(pixelCount));

    %% Set exposure parameters

    disp("Setting exposure parameters to: 20 ms, 10x averaging, no auto exposure");
    spectr.ExposureTime = 0.02; % in seconds
    spectr.Averaging = 10;
    spectr.AutoExposure = false;
 
    %% Start Exposure

    disp("Starting exposure ...")
    spectr.StartExposure(); % optional parameter: number of spectra

    %% Wait for exposure to be finished

    while (1)
        [status, availableSpectra] = spectr.GetStatus();
        if (status == DeviceClasses.SpectrStatus.Idle)
            break;
        end
        pause(0.01);
    end
    fprintf('Exposure finished. %d spectrum available in device.\n', availableSpectra)
    
    %% Read spectrum from device

    disp("Reading spectrum from device.")
    spectrumData = spectr.GetSpectrumData();
    values = spectrumData.Values;

    % Now we have the full spectrum in the spectrumData class. It also includes some metadata, for example:
    fprintf("Load level: %.2f %\n", spectrumData.LoadLevel * 100);

    %% Display values in a plot

    plot(wavelengths, values);
    grid on
    axis tight
    xlabel('Wavelength [nm]')
    ylabel('Intensity')
    title('Measured Spectrum')

catch e
    e.message
    if(isa(e,'NET.NetException'))
    e.ExceptionObject
    end
    disp('Cannot take spectrum')
end

%% Close connection to device

spectr.Close();
disp('Done')